// 002 u_c_analyze_db
// 03 feb 2005

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_analyze_db;
  interface
    uses u_c_basic_object
         , u_c_dfm_object
         , u_c_dataset_list
         ;

    type c_analyze_db= class(c_basic_object)
                         m_c_dataset_list_ref: c_dataset_list;
                         Constructor create_analyze_db(p_name: String);
                         procedure analyze(p_c_dfm_object: c_dfm_object);
                         Destructor Destroy; Override;
                       end; // c_analyze_db

  implementation
    uses SysUtils
        , db
        , u_c_display
        , u_dataset
        , u_c_dataset_kinds
        ;

    // -- c_analyze_db

    Constructor c_analyze_db.create_analyze_db(p_name: String);
      begin
        Inherited create_basic_object(p_name);
      end; // create_analyze_db

    procedure c_analyze_db.analyze(p_c_dfm_object: c_dfm_object);

      procedure analyze_recursive(p_level: Integer; p_c_dfm_object: c_dfm_object);

        function f_string_to_integer(p_string: String): Integer;
          begin
            if p_string= ''
              then Result:= 0
              else Result:= StrToInt(p_string);
          end; // f_string_to_integer

        procedure add_fields(p_c_dfm_object_list: c_dfm_object_list; p_c_dataset: c_dataset);
          var l_dfm_object_index: Integer;
              l_field_type: tFieldType;
          begin
            with p_c_dfm_object_list do
              for l_dfm_object_index:= 0 to f_dfm_object_count- 1 do
                with f_c_dfm_object(l_dfm_object_index) do
                begin
                  l_field_type:= f_field_type_name_to_field_type(m_name);

                  if l_field_type in [ftString]
                    then p_c_dataset.f_c_add_unique_field(
                       f_find_property_value('FieldName'),
                       l_field_type,
                       f_string_to_integer(f_find_property_value('Size')))
                    else p_c_dataset.f_c_add_unique_field(
                       f_find_property_value('FieldName'),
                       l_field_type, 0);
(*
    object Table1BMP: TBlobField
      FieldName = 'BMP'
      BlobType = ftTypedBinary
      Size = 1
    end
*)
                end;
          end; // add_fields

        procedure analyze_table;
          var // l_property_index: Integer;
              l_table_name: String;
              l_index_of_table: Integer;
              l_c_dataset_table: c_dataset_table;
          begin
            with p_c_dfm_object do
            begin
              display('table');

              l_table_name:= f_find_property_value('TableName');

              if l_table_name<> ''
                then begin
                    display('  '+ l_table_name);
                    l_index_of_table:= m_c_dataset_list_ref.f_index_of(l_table_name);
                    if l_index_of_table< 0
                      then begin
                          l_c_dataset_table:= c_dataset_table.create_dataset_table(l_table_name,
                              m_c_dataset_list_ref);
                          m_c_dataset_list_ref.add_dataset(l_table_name, l_c_dataset_table);
                        end
                      else l_c_dataset_table:= c_dataset_table(m_c_dataset_list_ref.f_c_dataset(l_index_of_table));

                    l_c_dataset_table.m_c_component_name_list.Add(m_object_name);
                    // stop(m_object_name);

                    // -- check whether "DataSource"
                    l_c_dataset_table.m_master_datasource:= f_find_property_value('MasterSource');
                    l_c_dataset_table.m_masterfields:= f_find_property_value('MasterFields');

                    add_fields(m_c_dfm_object_list_, l_c_dataset_table);
                  end
                else display('  NO_TABLE_NAME');
            end; // with p_c_dfm_object
          end; // analyze_table

        procedure analyze_query;
          var // l_property_index: Integer;
              l_request: String;
              l_index_of_query: Integer;
              l_c_dataset_query: c_dataset_query;
              // l_datasource: String;
          begin
            with p_c_dfm_object do
            begin
              display('query');

              l_request:= f_find_property_value('SQL.Strings');

              if l_request<> ''
                then begin
                    System.Delete(l_request, 1, 1);
                    System.Delete(l_request, Length(l_request), 1);
                    l_request:= Trim(l_request);

                    display('  '+ l_request);
                    l_index_of_query:= m_c_dataset_list_ref.f_index_of(l_request);
                    if l_index_of_query< 0
                      then begin
                          l_c_dataset_query:= c_dataset_query.create_dataset_query(l_request,
                              m_c_dataset_list_ref);
                          m_c_dataset_list_ref.add_dataset(l_request, l_c_dataset_query);
                        end
                      else l_c_dataset_query:= c_dataset_query(m_c_dataset_list_ref.f_c_dataset(l_index_of_query));

                    l_c_dataset_query.m_c_component_name_list.Add(m_object_name);
                    // -- check whether "DataSource"
                    l_c_dataset_query.m_master_datasource:= f_find_property_value('DataSource');

                    add_fields(m_c_dfm_object_list_, l_c_dataset_query);
                  end
                else display('  NO_SQL');
            end; // with p_c_dfm_object
          end; // analyze_query

        procedure recurse;
          var l_dfm_object_index: Integer;
              l_property_index: Integer;
          begin
            with p_c_dfm_object do
            begin
              with m_c_dfm_property_list do
                for l_property_index:= 0 to f_dfm_property_count- 1 do
                  with f_c_dfm_property(l_property_index) do
                  begin
                  end;

              with m_c_dfm_object_list_ do
                for l_dfm_object_index:= 0 to f_dfm_object_count- 1 do
                  analyze_recursive(p_level+ 1, f_c_dfm_object(l_dfm_object_index));
            end; // with p_c_dfm_object
          end; // recurse

        begin // analyze_recursive
          with p_c_dfm_object do
          begin
            if (LowerCase(m_name)= 'ttable') or (m_name= 'TIBTable')
              then analyze_table else
            if (m_name= 'TQuery') or (LowerCase(m_name)= 'tibquery')
              then analyze_query
              else recurse;
          end; // with p_c_dfm_object
        end; // analyze_recursive

      procedure resolve_master_detail;
          // -- if a tTable or tQuery reference a datasource, find its associated tDataset
        var l_dataset_index: Integer;
            l_c_dfm_datasource: c_dfm_object;
            l_master_dataset: String;
            l_c_dfm_master_dataset: c_dfm_object;
            l_dataset_name: String;
        begin
          display_line;

          with m_c_dataset_list_ref do
            for l_dataset_index:= 0 to f_dataset_count- 1 do
              with f_c_dataset(l_dataset_index) do
                if m_master_datasource<> ''
                  then begin
                      if not m_did_resolve
                        then begin
                            display('MAST '+ m_master_datasource);
                            // -- search this tDataSource in the .DFM
                            with p_c_dfm_object do
                            begin
                              l_c_dfm_datasource:= m_c_dfm_object_list_.f_c_find_by_dfm_object_name(m_master_datasource);
                              if l_c_dfm_datasource<> Nil
                                then begin
                                    // -- get the "DataSet" property value
                                    l_master_dataset:= l_c_dfm_datasource.f_find_property_value('DataSet');
                                    display('  found '+ l_master_dataset+ '<');

                                    // -- now find the Master Dataset
                                    l_c_dfm_master_dataset:= m_c_dfm_object_list_.f_c_find_by_dfm_object_name(l_master_dataset);
                                    if l_c_dfm_master_dataset<> Nil
                                      then begin
                                          l_dataset_name:= l_c_dfm_master_dataset.m_object_name;
                                          display('  OK '+ l_dataset_name);
                                          m_c_master_dataset_ref:= m_c_dataset_list_ref.f_c_find_by_dataset_component_name(l_dataset_name);
                                          if m_c_master_dataset_ref<> Nil
                                            then display('    OK')
                                            else begin
                                                display('    nok '+ '');
                                              end;
                                        end;
                                  end;
                            end;

                            // -- do not try to resolve in another form
                            m_did_resolve:= True;
                          end;
                    end;
        end; // resolve_master_detail

      procedure consolidate_queries;
        begin
        end; // consolidate_queries

      begin // analyze
        analyze_recursive(0, p_c_dfm_object);

        resolve_master_detail;
        consolidate_queries;
      end; // analyze

    Destructor c_analyze_db.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_analyze_db
    end. // u_c_analyze_db
