// 002 u_c_dataset_list
// 03 feb 2005

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- c_data
// --   c_dataset
// --     c_dataset_list

(*$r+*)

unit u_c_dataset_list;
  interface
    uses Classes, u_c_basic_object
        , db
        ;

    type c_field= // one "field"
                    Class(c_basic_object)
                      // -- m_name:
                      m_field_type: tFieldType;
                      m_field_size: Integer;

                      Constructor create_field(p_name: String;
                          p_field_type: tFieldType; p_field_size: Integer);
                      function f_display_field: String;
                      Destructor Destroy; Override;
                    end; // c_field

         c_dataset_list= Class; // forwar
         c_dataset= // one "dataset"
                    Class(c_basic_object)
                      // -- m_name: the Server NAME or the SQL request
                      m_c_parent_dataset_list: c_dataset_list;
                      m_c_field_list: tStringList;

                      // -- the component name (ASSUME only one has the same request)
                      // -- used to locate MasterDataset from MasterSource: only
                      // --   needs the value from the last analyzed tForm
                      // -- ^ NO
                      // -- a list, since one server Table can be references by
                      // --   several tDatasets in a tForm
                      m_c_component_name_list: tStringList;

                      // -- Master / Detail
                      m_master_datasource: String;
                      m_c_master_dataset_ref: c_dataset;
                      m_did_resolve: Boolean;

                      Constructor create_dataset(p_name: String;
                          p_c_parent_dataset_list: c_dataset_list);

                      function f_c_self: c_dataset;
                      function f_display_dataset: String;

                      function f_field_count: Integer;
                      function f_c_field(p_field_index: Integer): c_field;
                      function f_index_of(p_field_name: String): Integer;
                      function f_c_find_by_field(p_field_name: String): c_field;
                      procedure add_field(p_field_name: String; p_c_element: c_field);
                      function f_c_add_field(p_field_name: String;
                          p_field_type: tFieldType; p_field_size: Integer): c_field;
                      function f_c_add_unique_field(p_field_name: String;
                          p_field_type: tFieldType; p_field_size: Integer): c_field;
                      procedure display_field_list;

                      procedure display_detailed_dataset_fields;
                      procedure display_detailed_dataset; Virtual;

                      Destructor Destroy; Override;
                    end; // c_dataset

         c_dataset_list= // "dataset" list
                         Class(c_basic_object)
                           m_c_dataset_list: tStringList;

                           // -- detail display options
                           m_display_fields: Boolean;

                           Constructor create_dataset_list(p_name: String);

                           function f_dataset_count: Integer;
                           function f_c_dataset(p_dataset_index: Integer): c_dataset;
                           function f_index_of(p_dataset_index: String): Integer;
                           function f_c_find_by_dataset(p_dataset_index: String): c_dataset;
                           function f_c_find_by_dataset_component_name(p_dataset_component_name: String): c_dataset;
                           procedure add_dataset(p_dataset_index: String; p_c_dataset: c_dataset);
                           function f_c_add_dataset(p_dataset_index: String): c_dataset;
                           function f_c_add_unique_dataset(p_dataset_index: String): c_dataset;
                           procedure display_dataset_list;
                           procedure display_detailed_dataset_list;

                           procedure extract_sql(p_full_file_name: String);

                           Destructor Destroy; Override;
                         end; // c_dataset_list

  implementation
    uses SysUtils, u_c_display
      , u_dataset
      , u_c_dataset_kinds
      ;

    // -- c_field

    Constructor c_field.create_field(p_name: String;
        p_field_type: tFieldType; p_field_size: Integer);
      begin
        Inherited create_basic_object(p_name);

        m_field_type:= p_field_type;
        m_field_size:= p_field_size;
      end; // create_field

    function c_field.f_display_field: String;
      begin
        Result:= Format('%-10s ', [m_name]);
      end; // f_display_field

    Destructor c_field.Destroy;
      begin
        InHerited;
      end; // Destroy

    // -- c_dataset

    Constructor c_dataset.create_dataset(p_name: String;
        p_c_parent_dataset_list: c_dataset_list);
      begin
        Inherited create_basic_object(p_name);

        m_c_parent_dataset_list:= p_c_parent_dataset_list;
        m_c_field_list:= tStringList.Create;
        m_c_component_name_list:= tStringList.Create;
      end; // create_dataset

    function c_dataset.f_c_self: c_dataset;
      begin
        Result:= Self;
      end; // f_c_self
        
    function c_dataset.f_display_dataset: String;
      begin
        Result:= Format('%-10s ', [m_name]);
      end; // f_display_dataset

    function c_dataset.f_field_count: Integer;
      begin
        Result:= m_c_field_list.Count;
      end; // f_field_count

    function c_dataset.f_c_field(p_field_index: Integer): c_field;
      begin
        Result:= c_field(m_c_field_list.Objects[p_field_index]);
      end; //  f_c_field

    function c_dataset.f_index_of(p_field_name: String): Integer;
      begin
        Result:= m_c_field_list.IndexOf(p_field_name);
      end; // f_index_of

    function c_dataset.f_c_find_by_field(p_field_name: String): c_field;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_field_name);
        if l_index_of< 0
          then Result:= Nil
          else Result:= c_field(m_c_field_list.Objects[l_index_of]);
      end; // f_c_find_by_field

    procedure c_dataset.add_field(p_field_name: String; p_c_element: c_field);
      begin
        m_c_field_list.AddObject(p_field_name, p_c_element);
      end; // add_field

    function c_dataset.f_c_add_field(p_field_name: String;
        p_field_type: tFieldType; p_field_size: Integer): c_field;
      begin
        Result:= c_field.create_field(p_field_name, p_field_type, p_field_size);
        m_c_field_list.AddObject(p_field_name, Result);
      end; // f_c_add_field

    function c_dataset.f_c_add_unique_field(p_field_name: String;
        p_field_type: tFieldType; p_field_size: Integer): c_field;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_field_name);
        if l_index_of>= 0
          then Result:= f_c_field(l_index_of)
          else Result:= f_c_add_field(p_field_name, p_field_type, p_field_size);
      end; // add_unique_dataset

    procedure c_dataset.display_field_list;
      var l_field_index: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_field_count));

        for l_field_index:= 0 to f_field_count- 1 do
          display(f_c_field(l_field_index).f_display_field);
      end; // display_field_list

    procedure c_dataset.display_detailed_dataset_fields;
      var l_field_index: Integer;
      begin
        for l_field_index:= 0 to f_field_count- 1 do
          with f_c_field(l_field_index) do
            display(Format('  %-15s %-10s %3d', [m_name, f_field_type_name(m_field_type), m_field_size]));
      end; // display_detailed_dataset_fields

    procedure c_dataset.display_detailed_dataset;
      begin
      end; // display_detailed_dataset

    Destructor c_dataset.Destroy;
      var l_field_index: Integer;
      begin
        for l_field_index:= 0 to f_field_count- 1 do
          f_c_field(l_field_index).Free;
        m_c_field_list.Free;

        m_c_component_name_list.Free;

        InHerited;
      end; // Destroy

    // -- c_dataset_list

    Constructor c_dataset_list.create_dataset_list(p_name: String);
      begin
        Inherited create_basic_object(p_name);

        m_c_dataset_list:= tStringList.Create;
      end; // create_dataset_line

    function c_dataset_list.f_dataset_count: Integer;
      begin
        Result:= m_c_dataset_list.Count;
      end; // f_dataset_count

    function c_dataset_list.f_c_dataset(p_dataset_index: Integer): c_dataset;
      begin
        Result:= c_dataset(m_c_dataset_list.Objects[p_dataset_index]);
      end; //  f_c_dataset

    function c_dataset_list.f_index_of(p_dataset_index: String): Integer;
      begin
        Result:= m_c_dataset_list.IndexOf(p_dataset_index);
      end; // f_index_of

    function c_dataset_list.f_c_find_by_dataset(p_dataset_index: String): c_dataset;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_dataset_index);
        if l_index_of< 0
          then Result:= Nil
          else Result:= c_dataset(m_c_dataset_list.Objects[l_index_of]);
      end; // f_c_find_by_name

    function c_dataset_list.f_c_find_by_dataset_component_name(p_dataset_component_name: String): c_dataset;
      var l_index_of: Integer;
      begin
        Result:= Nil;
        for l_index_of:= 0 to f_dataset_count- 1 do
          with f_c_dataset(l_index_of) do
            if m_c_component_name_list.IndexOf(p_dataset_component_name)>= 0
              then begin
                  Result:= f_c_self;
                  Break;
                end;
      end; // f_c_find_by_dataset_component_name

    procedure c_dataset_list.add_dataset(p_dataset_index: String; p_c_dataset: c_dataset);
      begin
        m_c_dataset_list.AddObject(p_dataset_index, p_c_dataset);
      end; // add_dataset

    function c_dataset_list.f_c_add_dataset(p_dataset_index: String): c_dataset;
      begin
        Result:= c_dataset.create_dataset(p_dataset_index, Self);
        m_c_dataset_list.AddObject(p_dataset_index, Result);
      end; // f_c_add_dataset

    function c_dataset_list.f_c_add_unique_dataset(p_dataset_index: String): c_dataset;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_dataset_index);
        if l_index_of>= 0
          then Result:= f_c_dataset(l_index_of)
          else Result:= f_c_add_dataset(p_dataset_index);
      end; // add_unique_dataset

    procedure c_dataset_list.display_dataset_list;
      var l_dataset_index: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_dataset_count));

        for l_dataset_index:= 0 to f_dataset_count- 1 do
          display(f_c_dataset(l_dataset_index).f_display_dataset);
      end; // display_dataset_list

    procedure c_dataset_list.display_detailed_dataset_list;
      var l_dataset_index: Integer;
      begin
        display('> dataset_list');
        display(m_name+ ' '+ IntToStr(f_dataset_count));

        for l_dataset_index:= 0 to f_dataset_count- 1 do
          with f_c_dataset(l_dataset_index) do
            display_detailed_dataset;

        display('< dataset_list');
      end; // display_detailed_dataset_list

    procedure c_dataset_list.extract_sql(p_full_file_name: String);
      var l_dataset_index: Integer;
          l_c_sql_list: tStringList;
      begin
        display('> extract_sql');
        l_c_sql_list:= tStringList.Create;

        display(m_name+ ' '+ IntToStr(f_dataset_count));

        for l_dataset_index:= 0 to f_dataset_count- 1 do
          with f_c_dataset(l_dataset_index) do
            if f_c_self is c_dataset_query
              then l_c_sql_list.Add(m_name);

        l_c_sql_list.SaveToFile(p_full_file_name);
        l_c_sql_list.Free;

        display('< extract_sql');
      end; // extract_sql

    Destructor c_dataset_list.Destroy;
      var l_dataset_index: Integer;
      begin
        for l_dataset_index:= 0 to f_dataset_count- 1 do
          f_c_dataset(l_dataset_index).Free;
        m_c_dataset_list.Free;

        Inherited;
      end; // Destroy

    begin // u_c_dataset_list
    end. // u_c_dataset_list
