// 002 u_c_dfm_scanner
// 03 feb 2005

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_dfm_scanner;
  interface
    uses Classes, u_types_constants, u_c_text_file;

    type t_dfm_symbol_type= (e_unknown_symbol,
                            e_integer_symbol,
                            e_double_symbol,
                            e_string_litteral_symbol,

                            e_colon_symbol,
                            e_equal_symbol,
                            e_point_symbol,
                            e_comma_symbol,
                            e_plus_symbol,
                            e_opening_parenthesis_symbol, e_closing_parenthesis_symbol,
                            e_opening_bracket_symbol, e_closing_bracket_symbol,
                            e_opening_brace_symbol, e_closing_brace_symbol,
                            e_lower_symbol, e_greater_symbol,

                            // e_operator_symbol,

                            e_identifier_symbol,
                            e_object_symbol,
                            e_end_symbol,
                            e_TRUE_symbol, e_FALSE_symbol,

                            e_end_of_parse_symbol,
                            // -- for the generation
                            e_class_symbol
                         );

         c_dfm_scanner= class(c_text_file)
                          m_dfm_symbol_type: t_dfm_symbol_type;
                          m_blank_string, m_symbol_string: String;
                          m_display_symbol_read: Boolean;
                          // -- accept as number values starting with A..Z
                          m_in_brace: Boolean;

                          constructor create_dfm_scanner(p_name, p_file_name: String);
                          function f_initialized: Boolean;
                          function f_read_symbol: Boolean;
                          procedure test_dfm_scanner;
                          procedure scan_and_save(p_full_file_name: String);
                          destructor Destroy; Override;
                        end; // c_dfm_scanner

    function f_dfm_symbol_name(p_dfm_symbol_type: t_dfm_symbol_type): String;
    function f_symbol_to_type(p_symbol_type_name: String): String;

  implementation
    uses SysUtils, Dialogs, TypInfo, u_c_display;

    function f_dfm_symbol_name(p_dfm_symbol_type: t_dfm_symbol_type): String;
      begin
        Result:= GetEnumName(TypeInfo(t_dfm_symbol_type), Integer(p_dfm_symbol_type));
        Delete(Result, 1, 2);
        Delete(Result, Length(Result)- Length('_symbol')+ 1, Length('_symbol'));
      end; // f_dfm_symbol_name

    function f_symbol_to_type(p_symbol_type_name: String): String;
      begin
        Result:= '';

        if p_symbol_type_name= 'integer'
          then Result:= 'Integer' else
        if p_symbol_type_name= 'double'
          then Result:= 'Double' else
        if p_symbol_type_name= 'string_litteral'
          then Result:= 'String' else
        if (p_symbol_type_name= 'TRUE') or (p_symbol_type_name= 'FALSE')
          then Result:= 'Boolean' else
        if p_symbol_type_name= 'identifier'
          then Result:= 't_xxx_type' else
        if p_symbol_type_name= 'class'
          then Result:= 'xxx_class' else
      end; // f_symbol_to_type

    constructor c_dfm_scanner.create_dfm_scanner(p_name, p_file_name: String);
      begin
        Inherited create_text_file(p_name, p_file_name);
      end; // create_dfm_scanner

    function c_dfm_scanner.f_initialized: Boolean;
      begin
        if f_load
          then begin
              Result:= true;
              m_buffer_index:= 0;
            end
          else begin
              display('*** could not open '+ m_c_file_name.m_complete_name
                  + ' '+ IntToStr(m_io_result));
              Result:= False;
            end;
      end; // f_initialized

    function c_dfm_scanner.f_read_symbol: Boolean;
        // -- true until eof
      const k_blanks= [' ', chr(9)];
            k_returns= [chr(13), chr(10)];
            k_blanks_returns= k_blanks+ k_returns;
            k_digits= ['0'..'9'];
            k_hex_digits= ['A'..'F'];
            k_identifier= ['a'..'z', 'A'..'Z', '_', '0'..'9'];
            k_other= k_blanks_returns+ k_identifier;

      procedure get_blanks;
        var l_start: Integer;
        begin
          l_start:= m_buffer_index;
          while (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index] in k_blanks_returns) do
            inc(m_buffer_index);
          m_blank_string:= f_extract_string_start_end(l_start, m_buffer_index- 1);
        end; // get_blanks

      procedure get_identifier;
        var l_start: Integer;
        begin
          m_dfm_symbol_type:= e_identifier_symbol;
          l_start:= m_buffer_index;
          while (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index] in k_identifier) do
            inc(m_buffer_index);

          m_symbol_string:= f_extract_string_start_end(l_start, m_buffer_index- 1);

          if m_symbol_string= 'object'
            then m_dfm_symbol_type:= e_object_symbol else
          if m_symbol_string= 'end'
            then m_dfm_symbol_type:= e_end_symbol else
          if LowerCase(m_symbol_string)= 'true'
            then m_dfm_symbol_type:= e_true_symbol else
          if LowerCase(m_symbol_string)= 'false'
            then m_dfm_symbol_type:= e_false_symbol else
        end; // get_identifier

      procedure get_number;
          // -- can have hex, exponent ?
        var l_start: Integer;
        begin
          m_dfm_symbol_type:= e_integer_symbol;
          l_start:= m_buffer_index;
          // -- skip the possible "-"
          Inc(m_buffer_index);
          while (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index] in k_digits) do
            inc(m_buffer_index);

          if (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index]= '.')
            then begin
                m_dfm_symbol_type:= e_double_symbol;
                Inc(m_buffer_index);
                while (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index] in k_digits) do
                  inc(m_buffer_index);
              end
            else
              if (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index] in k_hex_digits)
                then begin
                    Inc(m_buffer_index);
                    while (m_buffer_index< m_buffer_size)
                        and (m_pt_buffer[m_buffer_index] in (k_digits+ k_hex_digits)) do
                      inc(m_buffer_index);
                  end
                else
                  if (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index]= 'd')
                    then begin
                        // -- IbMastApp "3477d"
                        Inc(m_buffer_index);
                      end;

          m_symbol_string:= f_extract_string_start_end(l_start, m_buffer_index- 1);
        end; // get_number

      procedure get_one_operator(p_symbol_type: t_dfm_symbol_type);
        begin
          m_dfm_symbol_type:= p_symbol_type;
          m_symbol_string:= m_pt_buffer[m_buffer_index];
          Inc(m_buffer_index);
        end; // get_one_operator

      procedure get_string_litteral;
        // -- "Caption = 'Copyright '#169' 1995-2005 my_company.'"
        // -- "Caption = #174"

        procedure get_quoted_string;
          begin
            repeat
              repeat
                inc(m_buffer_index);
              until m_pt_buffer[m_buffer_index]= '''';

              // -- get rid of double ''
              inc(m_buffer_index);
            until m_pt_buffer[m_buffer_index]<> '''';
          end; // get_quoted_string

        procedure get_diese_character;
          begin
            // -- skip #
            Inc(m_buffer_index);
            while (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index] in k_digits) do
              inc(m_buffer_index);
          end; // get_diese_character

        var l_start: Integer;

        begin // get_string_litteral
          m_dfm_symbol_type:= e_string_litteral_symbol;
          l_start:= m_buffer_index;

          repeat
            // display('str >'+ m_pt_buffer[m_buffer_index]+ '<');
            case m_pt_buffer[m_buffer_index] of
              '''' : get_quoted_string;
              '#' : get_diese_character;
            end;
          until not (m_pt_buffer[m_buffer_index] in ['''', '#']);
          // display('aft >'+ m_pt_buffer[m_buffer_index]+ '<');

          // -- ?? check -1
          m_symbol_string:= f_extract_string_start_end(l_start, m_buffer_index- 1);
        end; // get_string_litteral

      var // -- for display
          l_start_index: Integer;

      begin // f_read_symbol
        m_symbol_string:= '';
        m_dfm_symbol_type:= e_unknown_symbol;
        l_start_index:= m_buffer_index;

        if f_end_of_text
          then begin
              Result:= False;
              m_dfm_symbol_type:= e_end_of_parse_symbol;
            end
          else begin
              get_blanks;

              l_start_index:= m_buffer_index;

              if m_buffer_index< m_buffer_size
                then begin
                    case m_pt_buffer[m_buffer_index] of
                      'a'..'z', 'A'..'Z' :
                          if m_in_brace and (m_pt_buffer[m_buffer_index] in k_hex_digits)
                            then get_number
                            else get_identifier;
                      '-', '0'..'9' : get_number;
                      ':' : get_one_operator(e_colon_symbol);
                      '.' : get_one_operator(e_point_symbol);
                      '=' : get_one_operator(e_equal_symbol);
                      ',' : get_one_operator(e_comma_symbol);
                      '+' : get_one_operator(e_plus_symbol);
                      '''', '#': get_string_litteral;
                      '(' : get_one_operator(e_opening_parenthesis_symbol);
                      ')' : get_one_operator(e_closing_parenthesis_symbol);
                      '[' : get_one_operator(e_opening_bracket_symbol);
                      ']' : get_one_operator(e_closing_bracket_symbol);
                      '{' : begin
                              m_in_brace:= True;
                              get_one_operator(e_opening_brace_symbol);
                            end;
                      '}' : begin
                              get_one_operator(e_closing_brace_symbol);
                              m_in_brace:= False;
                            end;
                      '<' : get_one_operator(e_lower_symbol);
                      '>' : get_one_operator(e_greater_symbol);
                      else
                        display_bug_stop('unknown_char_in_dfm '+ m_pt_buffer[m_buffer_index]
                            + '< '+ IntToStr(Ord(m_pt_buffer[m_buffer_index])));
                    end; // case
                  end
                else m_dfm_symbol_type:= e_end_of_parse_symbol;

              Result:= True;
            end; // not eof

        if m_display_symbol_read
          then begin
              display(Format('%5d ', [l_start_index])+ m_symbol_string+ ' '+ f_dfm_symbol_name(m_dfm_symbol_type));
            end;
      end; // f_read_symbol

    procedure c_dfm_scanner.test_dfm_scanner;
      begin
        if f_initialized
          then begin
              m_buffer_index:= 0;
              repeat
                f_read_symbol;
              until m_dfm_symbol_type= e_end_of_parse_symbol;
            end;
      end; // test_dfm_scanner

    procedure c_dfm_scanner.scan_and_save(p_full_file_name: String);
      var l_c_result_text: tStringList;
      begin
        if f_initialized
          then begin
              l_c_result_text:= tStringList.Create;
              m_buffer_index:= 0;
              repeat
                f_read_symbol;
                if m_symbol_string<> ''
                  then l_c_result_text.Add(m_symbol_string);
              until m_dfm_symbol_type= e_end_of_parse_symbol;

              l_c_result_text.SaveToFile(p_full_file_name);
              l_c_result_text.Free;
            end;
      end; // scan_and_save

    destructor c_dfm_scanner.Destroy;
      begin
        Inherited Destroy;
      end; // Destroy

    begin // c_dfm_scanner
    end. // c_dfm_scanner


